@@ -112,7 +112,7 @@ static int bcrypt_verify(const char *password, const char* hash)
 
     int status = 0;
     int i = 0;
-    for(i; i < strlen(ret); i++)
+    for(; i < strlen(ret); i++)
         status |= (ret[i] ^ hash[i]);
     return status != 0;
 }
@@ -392,7 +392,8 @@ static int _ar_pgsql_check_password(authreg_t ar, sess_t sess, const char *usern
             if(ret == 0) {
                 if (bcrypt_needs_rehash(ctx->bcrypt_cost, db_pw_value)) {
                     char tmp[257];
-                    strcpy(tmp, password);
+                    strncpy(tmp, password, 256);
+                    tmp[256] = 0;
                     _ar_pgsql_set_password(ar, sess, username, realm, tmp);
                 }
             }
@@ -589,6 +590,7 @@ int ar_init(authreg_t ar) {
     int strlentur; /* string length of table, user, and realm strings */
     PGconn *conn;
     pgsqlcontext_t pgsqlcontext;
+    int fail = 0;
 
     /* configure the database context with field names and SQL statements */
     pgsqlcontext = (pgsqlcontext_t) calloc(1, sizeof( struct pgsqlcontext_st ) );
@@ -622,7 +624,7 @@ int ar_init(authreg_t ar) {
     } else if (config_get_one(ar->c2s->config, "authreg.pgsql.password_type.bcrypt", 0)) {
         pgsqlcontext->password_type = MPC_BCRYPT;
         int cost;
-        if(cost = j_atoi(config_get_attr(ar->c2s->config, "authreg.pgsql.password_type.bcrypt", 0, "cost"), 0))
+        if((cost = j_atoi(config_get_attr(ar->c2s->config, "authreg.pgsql.password_type.bcrypt", 0, "cost"), 0)))
         {
             if (cost < 4 || cost > 31) {
                 log_write(ar->c2s->log, LOG_ERR, "bcrypt cost has to be higher than 3 and lower than 32.");
@@ -641,7 +643,7 @@ int ar_init(authreg_t ar) {
     /* bounds checking and parameter format verification will be perfomed if the statement is used (see next section) */
     /* For malloc(), there is no +1 for trailing 0 as parameter substitution will net us several extra characters */
 
-    strlentur = strlen( table ) + strlen( username) + strlen( realm );  /* avoid repetition */
+    strlentur = strlen( table ) + strlen( username ) + strlen( realm );  /* avoid repetition */
 
     template = "INSERT INTO \"%s\" ( \"%s\", \"%s\" ) VALUES ( '%%s', '%%s' )";
     create = malloc( strlen( template ) + strlentur );
@@ -667,22 +669,22 @@ int ar_init(authreg_t ar) {
     pgsqlcontext->sql_create = strdup(_ar_pgsql_param( ar->c2s->config
            , "authreg.pgsql.sql.create"
            , create ));
-    if( _ar_pgsql_check_sql( ar, pgsqlcontext->sql_create, "ss" ) != 0 ) return 1;
+    if( _ar_pgsql_check_sql( ar, pgsqlcontext->sql_create, "ss" ) != 0 ) fail = 1;
 
     pgsqlcontext->sql_select = strdup(_ar_pgsql_param( ar->c2s->config
            , "authreg.pgsql.sql.select"
            , select ));
-    if( _ar_pgsql_check_sql( ar, pgsqlcontext->sql_select, "ss" ) != 0 ) return 1;
+    if( _ar_pgsql_check_sql( ar, pgsqlcontext->sql_select, "ss" ) != 0 ) fail = 1;
 
     pgsqlcontext->sql_setpassword = strdup(_ar_pgsql_param( ar->c2s->config
            , "authreg.pgsql.sql.setpassword"
            , setpassword ));
-    if( _ar_pgsql_check_sql( ar, pgsqlcontext->sql_setpassword, "sss" ) != 0 ) return 1;
+    if( _ar_pgsql_check_sql( ar, pgsqlcontext->sql_setpassword, "sss" ) != 0 ) fail = 1;
 
     pgsqlcontext->sql_delete = strdup(_ar_pgsql_param( ar->c2s->config
            , "authreg.pgsql.sql.delete"
            , delete ));
-    if( _ar_pgsql_check_sql( ar, pgsqlcontext->sql_delete, "ss" ) != 0 ) return 1;
+    if( _ar_pgsql_check_sql( ar, pgsqlcontext->sql_delete, "ss" ) != 0 ) fail = 1;
 
     // Check password is optional
     const char *sql_check_password = _ar_pgsql_param( ar->c2s->config, "authreg.pgsql.sql.checkpassword", 0);
@@ -690,7 +692,7 @@ int ar_init(authreg_t ar) {
     if (sql_check_password) {
         ar->check_password = _ar_pgsql_dbcheck_password;
         pgsqlcontext->sql_check_password = strdup(sql_check_password);
-        if( _ar_pgsql_check_sql( ar, pgsqlcontext->sql_check_password, "sss" ) != 0 ) return 1;
+        if( _ar_pgsql_check_sql( ar, pgsqlcontext->sql_check_password, "sss" ) != 0 ) fail = 1;
     }
     else
         ar->check_password = _ar_pgsql_check_password;
@@ -707,6 +709,8 @@ int ar_init(authreg_t ar) {
     free(setpassword);
     free(delete);
 
+    if (fail) return fail;
+
 #ifdef HAVE_SSL
     if(sx_openssl_initialized)
     PQinitSSL(0);
