@@ -159,7 +159,7 @@ typedef struct _sx_compress_conn_st {
 JABBERD2_API int sx_ack_init(sx_env_t env, sx_plugin_t p, va_list args);
 
 /* websocket wrapper plugin */
-//#ifdef USE_WEBSOCKET
+#ifdef USE_WEBSOCKET
 #include <http_parser.h>
 #include <util/util.h>
 
@@ -182,8 +182,11 @@ typedef struct _sx_websocket_conn_st {
     spool                   field, value;
     xht                     headers;
     void                    *frame;
+    unsigned int            opcode;
+    char                    *buf;
+    size_t                  buf_len;
 } *_sx_websocket_conn_t;
-//#endif
+#endif
 
 #ifdef __cplusplus
 }
