@@ -146,6 +146,7 @@ struct _sx_buf_st {
 #define stream_err_UNSUPPORTED_VERSION      (22)
 #define stream_err_XML_NOT_WELL_FORMED      (23)
 #define stream_err_LAST                     (24)
+/** if you change these, reflect your changes in the table in error.c */
 
 /* exported functions */
 
@@ -256,12 +257,12 @@ struct _sx_st {
     /* tag, for logging */
     int                      tag;
 
-	/* IP address of the connection */
-	/* pointing to sess.ip and owned by sess structure */
-	const char              *ip;
+    /* IP address of the connection */
+    /* pointing to sess.ip and owned by sess structure */
+    const char              *ip;
 
-	/* TCP port of the connection */
-	/* pointing to sess.port and owned by sess structure */
+    /* TCP port of the connection */
+    /* pointing to sess.port and owned by sess structure */
     int                     port;
 
     /* callback */
@@ -306,7 +307,10 @@ struct _sx_st {
 
     /* bytes read from socket */
     int                      rbytes;
-    int                      rbytes_total;
+    /* bytes parsed by parser */
+    int                      pbytes;
+    /* total bytes processed */
+    int                      tbytes;
 
     /* read bytes maximum */
     int                      rbytesmax;
